import sys
import shutil
import tempfile

import pytest

from webtest import TestApp

from quixote.wsgi import QWIP
from quixote import cleanup, get_publisher
from wcs.qommon import errors, sessions
from qommon.ident.password_accounts import PasswordAccount
from wcs import publisher
from wcs.qommon.http_request import HTTPRequest

QWIP.request_class = HTTPRequest


def setup_module(module):
    cleanup()

    global pub

    publisher.WcsPublisher.APP_DIR = tempfile.mkdtemp()
    pub = publisher.WcsPublisher.create_publisher()
    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.write_cfg()

def create_superuser():
    global user1
    user1 = pub.user_class(name='admin')
    user1.id = 'admin'
    user1.is_admin = True
    user1.store()

    account1 = PasswordAccount(id='admin')
    account1.set_password('admin')
    account1.user_id = user1.id
    account1.store()

def get_app():
    return TestApp(QWIP(pub), extra_environ={'HTTP_HOST': 'example.net'})

def login(app):
    login_page = app.get('/login/')
    login_form = login_page.forms['login-form']
    login_form['username'] = 'admin'
    login_form['password'] = 'admin'
    resp = login_form.submit()
    assert resp.status_int == 302
    return app


def teardown_module(module):
    shutil.rmtree(pub.APP_DIR)

def test_empty_site():
    get_app().get('/admin/')

def test_with_user():
    create_superuser()
    resp = get_app().get('/admin/', status=302)
    resp = resp.follow()
    assert resp.location == 'http://example.net/login/'

def test_with_superuser():
    app = login(get_app())
    app.get('/admin/')

def test_forms():
    app = login(get_app())
    app.get('/admin/forms/')
    app.get('/admin/forms/new')

def test_workflows():
    app = login(get_app())
    app.get('/admin/workflows/')
    app.get('/admin/workflows/new')

def test_users():
    app = login(get_app())
    app.get('/admin/users/')
    app.get('/admin/users/new')

def test_roles():
    app = login(get_app())
    app.get('/admin/roles/')
    app.get('/admin/roles/new')

def test_categories():
    app = login(get_app())
    app.get('/admin/categories/')
    app.get('/admin/categories/new')

def test_settings():
    app = login(get_app())
    app.get('/admin/settings/')
    app.get('/admin/settings/themes')
    app.get('/admin/settings/users')
    app.get('/admin/settings/template')
    app.get('/admin/settings/misc')
    app.get('/admin/settings/emails')
    app.get('/admin/settings/debug_options')
    app.get('/admin/settings/language')
    app.get('/admin/settings/import')
    app.get('/admin/settings/export')
    app.get('/admin/settings/identification')
    app.get('/admin/settings/sitename')
    app.get('/admin/settings/sms')
    app.get('/admin/settings/certificates')
    app.get('/admin/settings/texts')
    app.get('/admin/settings/session')
    app.get('/admin/settings/admin-permissions')
